def get_metrics(response):
    """
    Extract asked metrics from api response
    @list_metrics : list of dict
    """
    list_metrics = []
    for i in response['reports'][0]['columnHeader']['metricHeader']['metricHeaderEntries']:
        list_metrics.append(i['name'])
    return list_metrics


def get_dimensions(response):
    """
      Extract asked dimensions from api response
      @list_dimensions : list of dict
    """
    return response['reports'][0]['columnHeader']['dimensions']


def extract_api_data(response):
    """
    Extract all data from api response
    """
    try:
        rows = response['reports'][0]['data']['rows']
    except:
        return []
    try:
        samples_read_counts = response['reports'][0]['data']['samplesReadCounts']
        sampling_space_sizes = response['reports'][0]['data']['samplesReadCounts']
        print("SAMPLING")
        print(samples_read_counts)
        print(sampling_space_sizes)
        exit()
    except:
        pass

    metric_response = get_metrics(response)
    dimensions_response = get_dimensions(response)
    data = []
    for row in rows:
        d = {}
        j = 0
        for i in dimensions_response:
            d[i] = row['dimensions'][j]
            j = j + 1
        j = 0
        for i in metric_response:
            d[i] = row['metrics'][0]['values'][j]
            j = j + 1

        data.append(d)
    return data



mapping = {
    "ga:bounces": "bounces",
    "ga:yearMonth": "month",
    "ga:year": "year",
    "ga:date": "day",
    "ga:users": "users",
    "ga:region": "region",
    "ga:userAgeBracket": "age",
    "ga:userGender": "gender",
    "ga:sessions": "session",
    "ga:pageviews": "page_view",
    "ga:avgSessionDuration": "avg_session_duration",
    "ga:deviceCategory": "device",
    "ga:regionIsoCode": "region_iso_code",
    "ga:regionId": "region_id",
    "ga:adCost": 'ad_cost',
    "ga:impressions": "impression",
    "ga:adClicks": "click",
    "ga:adGroup": "ad_group",
    "ga:newUsers": "new_users",
    "ga:percentNewSessions": "percent_new_session",
    "ga:pagePathLevel1": "content_group",
    "ga:acquisitionTrafficChannel": "channel",
    "ga:landingPagePath": "landing",
    "ga:exitPagePath": "exit",
    "ga:channelGrouping": "channel_grouping",
    "ga:pagePath": 'page_path',
    "ga:pageTitle": 'page_title',
    "ga:country": 'country',
    "ga:isoYearIsoWeek": "week"
}



import copy
from pyganalytics import MetaGoogleAnalytics
from pyganalytics.core.extract.api import get_report
from pyganalytics.core.transform.extract import extract_api_data
from pyganalytics.core.transform.path import mapping_path
import hashlib
from isoweek import Week


def create_id(view_id, date):
    a = str(view_id) + str(date)
    id_batch = hashlib.sha224(a.encode()).hexdigest()
    return id_batch


def treat_data(data, metric, dimension):
    """
    Treat Data:
    -Cluster sources
    -Date and metric format
    """
    for i in data:
        for j in metric:
            i[j] = float(i[j])
        for d in dimension:
            if d == "ga:yearMonth":
                i["date"] = i[d][:4] + "-" + i[d][-2:] + "-01"
            elif d == "ga:year":
                i["date"] = i[d] + "-01-01"
            elif d == "ga:isoYearIsoWeek":
                week = Week(int(i[d][:4]), int(i[d][-2:])).monday()
                i["date"] = str(week)[:10]
            elif d == "ga:date":
                i["date"] = i[d][:4] + "-" + i[d][4:6] + "-" + i[d][6:8]
            if len(i[d]) > 254:
                i[d] = i[d][:254]
    return data


def get_data(googleanalytics: MetaGoogleAnalytics, view_id, start, end, metric, dimension, time_increment,
             metric_filter=None,
             dimension_filter=None, segment=None):
    print(view_id)
    mapping_reverse = mapping_path(googleanalytics)[1]
    dimension = copy.deepcopy(dimension)

    for d in range(len(dimension)):
        try:
            dimension[d] = mapping_reverse[dimension[d]]
        except KeyError:
            pass

    try:
        dimension.append(mapping_reverse[time_increment])
    except KeyError:
        dimension.append(time_increment)
    analytics = googleanalytics.googleauthentication.get_account("analyticsreporting")

    if metric_filter is not None:
        metric_filter = [
            {
                "filters": metric_filter
            }]
    if dimension_filter is not None:
        dimension_filter = [
            {
                "filters": dimension_filter
            }
        ]
    if segment:
        segment = [{"segmentId": segment}]

    response = get_report(analytics, view_id, dimension, metric, start, end, metric_filter=metric_filter,
                          dimension_filter=dimension_filter, segments=segment)

    next_page_token = response['reports'][0].get("nextPageToken")

    api_data = extract_api_data(response)
    data = treat_data(api_data, metric, dimension)

    while next_page_token is not None:
        response = get_report(analytics, view_id, dimension, metric, start, end, metric_filter=metric_filter,
                              dimension_filter=dimension_filter, segments=segment, page_token=next_page_token)

        next_page_token = response['reports'][0].get("nextPageToken")

        api_data = extract_api_data(response)
        data = data + treat_data(api_data, metric, dimension)
    return data


def create_columns_rows(googleanalytics: MetaGoogleAnalytics, data, view_id, time_increment):
    mapping = mapping_path(googleanalytics)[0]
    try:
        column_set = data[0].keys()
    except IndexError:
        column_set = []
    column_dict = {}
    column_name = []
    for c in column_set:
        try:
            column_dict[mapping[c]] = c
            column_name.append(mapping[c])
        except KeyError:
            column_dict[c] = c
            column_name.append(c)
    rows = []
    all_batch_id = []
    for element in data:
        row = []
        batch_id = create_id(view_id, element[column_dict[time_increment]])

        for c in column_name:
            row.append(element[column_dict[c]])

        row.append(view_id)
        row.append(batch_id)
        rows.append(row)
        if batch_id not in all_batch_id:
            all_batch_id.append(batch_id)
    column_name.append("view_id")
    column_name.append("batch_id")
    result = {
        "columns_name": column_name,
        "rows": rows
    }
    return result, all_batch_id



import copy

from dbstream import DBStream


def send_to_db(dbstream: DBStream, data, all_batch_id):
    all_batch_id = ["'" + e + "'" for e in all_batch_id]
    table_name = data["table_name"]
    if all_batch_id:
        try:
            query = "DELETE FROM %s WHERE batch_id IN (%s);" % (table_name, ",".join(all_batch_id))
            dbstream.execute_query(query)
        except:
            print("Table does not exist")
    data["columns_name"] = [r.replace(":", "_").lower() for r in data["columns_name"]]
    copy_data = copy.deepcopy(data)
    total_length = len(copy.deepcopy(data)["rows"])
    try:
        dbstream.send_data(data, replace=False)
    except:
        i = 0
        while i < total_length:
            small_rows = copy_data["rows"][i:i + 50]
            print(small_rows)
            dbstream.send_data({
                "table_name": copy_data["table_name"],
                "columns_name": copy_data["columns_name"],
                "rows": small_rows
            }, replace=False)
            i = i + 50



metric_dimension_example = {
    'traffic': {
        'metric': [
            'ga:sessions',
            'ga:users',
            'ga:pageviews',
            'ga:avgSessionDuration',
            'ga:percentNewSessions',
            'ga:bounces'
        ],
        'dimension': [
            'ga:country'
        ],
        'metric_filter': {
            'metricName': 'ga:users',
            'operator': 'GREATER_THAN',
            'comparisonValue': '100'
        },
        'time_increment': [
            'week',
            'day'
        ]
    }
}



import time
import copy

from pyganalytics.MetaGoogleAnalytics import MetaGoogleAnalytics
from pyganalytics.core.load.to_database import send_to_db
from pyganalytics.core.extract.config import get_start_end, get_all_view_id
from pyganalytics.core.transform.path import get_metric_dimension
from pyganalytics.core.transform.core import get_data, create_columns_rows
from pyganalytics.core.extract.date import segment_month_date, segment_ndays_date


def _get_one_segment(googleanalytics, report, all_view_id, start, end, time_increment, prefix_schema):
    report_name = report.get("name")
    report_config = report.get("config")
    metric = list(report_config["metric"])
    dimension = list(report_config["dimension"])
    metric_filter = report_config.get("metric_filter")
    dimension_filter = report_config.get("dimension_filter")
    segment = report_config.get("segment")
    output_storage_name = "ga." + report_name + "_" + time_increment.replace(":", "_")
    if prefix_schema:
        output_storage_name = prefix_schema + "_" + output_storage_name

    result = {
        "rows": []
    }
    all_batch_id = []
    print("Loading " + report_name + " " + time_increment + " between " + start + " and " + end)
    for view_id in all_view_id:

        data = get_data(
            googleanalytics,
            view_id,
            start,
            end,
            metric,
            dimension,
            time_increment,
            metric_filter,
            dimension_filter,
            segment)

        view_result, view_all_batch_id = create_columns_rows(googleanalytics, data, view_id, time_increment)
        if result.get("columns_name") is None or (len(view_result["columns_name"]) > len(result["columns_name"])):
            result["columns_name"] = view_result["columns_name"]
        if len(view_result["rows"]) > 0 and len(view_result["rows"][0]) > 2:
            result["rows"] = result["rows"] + view_result["rows"]
        all_batch_id = all_batch_id + view_all_batch_id

    len_result = str(len(result["rows"]))
    print(time_increment + ": " + len_result + " results")

    result["table_name"] = output_storage_name
    copy_result = copy.deepcopy(result)
    send_to_db(dbstream=googleanalytics.dbstream, data=copy_result, all_batch_id=all_batch_id)
    print("Finished sent to Database " + report_name + " " + time_increment + " between " + start + " and " + end)

    return result


def _get_data_by_segment(googleanalytics, start, end, report, all_view_id, increment, prefix_schema):
    all_time_increment = report.get("config")["time_increment"]
    all_result = []
    for time_increment in all_time_increment:
        if time_increment == 'year':
            result = _get_one_segment(googleanalytics, report, all_view_id, start, end, time_increment, prefix_schema)
        else:
            if time_increment == 'day':
                segments = segment_ndays_date(start, end, increment)
            else:
                segments = segment_month_date(start, end)
            i = 0
            for segment in segments:
                segment_data = _get_one_segment(googleanalytics, report, all_view_id, segment[0], segment[1],
                                                time_increment, prefix_schema)
                if i == 0:  # Concatenate to send to spreadsheet
                    result = segment_data
                    i = i + 1
                else:
                    result["rows"] = result["rows"] + segment_data["rows"]
                time.sleep(2)
        all_result.append(result)
    return all_result


class GoogleAnalytics(MetaGoogleAnalytics):

    def get(self,
            start=None,
            end=None,
            all_view_id=None,
            prefix_schema=None,
            increment=5,
            return_result=False):
        metric_dimension = get_metric_dimension(self)
        start, end = get_start_end(start, end)
        all_view_id = get_all_view_id(self, all_view_id)

        for report_name in metric_dimension.keys():
            report = {
                "name": report_name,
                "config": metric_dimension[report_name]
            }
            print("Loading report %s" % report_name)
            all_result = _get_data_by_segment(self,
                                              start,
                                              end,
                                              report,
                                              all_view_id,
                                              increment,
                                              prefix_schema,
                                              )
            print("Finish loading report %s" % report_name)
            # time.sleep(5)
            if return_result:
                return all_result



def get_all_access_view(googleanalytics, only_account=False):
    analytics = googleanalytics.googleauthentication.get_account("analytics", version="v3")
    accounts = analytics.management().accounts().list().execute()
    all_view = []
    if only_account:
        return [{
            "account_id": a['id'],
            "account_name": a["name"]
        }
            for a in accounts['items']
        ]
    for a in accounts['items']:
        account_name = a['name']
        account_id = a['id']
        properties = analytics.management().webproperties().list(accountId=account_id).execute()
        for p in properties['items']:
            property_id = p['id']
            property_name = p['name']
            views = analytics.management().profiles().list(
                accountId=account_id,
                webPropertyId=property_id).execute()
            for v in views['items']:
                view_id = v['id']
                view_name = v['name']
                all_view.append(
                    {'account_name': account_name,
                     'account_id': account_id,
                     'property_name': property_name,
                     'property_id': property_id,
                     'view_id': view_id,
                     'view_name': view_name
                     })
    return all_view



from datetime import datetime, timedelta

DATE_FORMAT = "%Y-%m-%d"


def date_to_string(date):
    return date.strftime(DATE_FORMAT)


def string_to_date(string):
    return datetime.strptime(string, DATE_FORMAT)


def segment_month_date(start, end):
    """
    start : YYYY-MM-DD
    end : YYYY-MM-DD
    """
    start_datetime = string_to_date(start)
    end_datetime = string_to_date(end)
    result = []

    while start_datetime.replace(day=1) != end_datetime.replace(day=1):
        inter = [date_to_string(start_datetime), date_to_string(end_of_month(start_datetime))]
        result.append(inter)
        start_datetime = add_a_month(start_datetime)
    result.append([date_to_string(start_datetime), date_to_string(end_datetime)])
    return result


def segment_ndays_date(start, end, increment):
    """
    start : YYYY-MM-DD
    end : YYYY-MM-DD
    """
    start_datetime = string_to_date(start)
    end_datetime = string_to_date(end)
    result = []

    while (start_datetime + timedelta(days=increment)) < end_datetime:
        inter = [date_to_string(start_datetime), date_to_string(start_datetime + timedelta(days=increment))]
        result.append(inter)
        start_datetime = start_datetime + timedelta(days=increment + 1)
    result.append([date_to_string(start_datetime), date_to_string(end_datetime)])
    return result


def add_a_month(date):
    try:
        next_month = date.replace(month=date.month + 1, day=1)
    except ValueError:
        if date.month == 12:
            next_month = date.replace(year=date.year + 1, month=1, day=1)
        else:
            # next month is too short to have "same date"
            # pick your own heuristic, or re-raise the exception:
            raise
    return next_month


def remove_a_month(date):
    try:
        previous_month = date.replace(month=date.month - 1, day=1)
    except ValueError:
        if date.month == 1:
            previous_month = date.replace(year=date.year - 1, month=12, day=1)
        else:
            # next month is too short to have "same date"
            # pick your own heuristic, or re-raise the exception:
            raise
    return previous_month


def end_of_month(date):
    return add_a_month(date) + timedelta(days=-1)



import datetime
from pyganalytics.core.extract.manage_view import get_all_access_view


def get_start_end(start, end):
    if not end:
        end = str(datetime.datetime.now())[:10]
    if not start:
        start = str(datetime.datetime.strptime(end, "%Y-%m-%d") + datetime.timedelta(days=-10))[:10]
    return start, end


def get_all_view_id(googleanalytics, all_view_id):
    if all_view_id == "*":
        all_view_id = get_all_access_view(googleanalytics)
        all_view_id = [ai["view_id"] for ai in all_view_id]
    elif not all_view_id:
        print("No list of view_id given as argument")
        exit()
    return all_view_id



import json
import yaml
from pyganalytics.mapping import mapping


def mapping_path(googleanalytics):
    google_analytics_mapping_path = googleanalytics.mapping_path
    if google_analytics_mapping_path:
        custom_mapping = open(google_analytics_mapping_path).read()
        custom_mapping = json.loads(custom_mapping)
        mapping.update(custom_mapping)
    mapping_reverse = {}
    for x in mapping.keys():
        mapping_reverse[mapping[x]] = x
    return mapping, mapping_reverse


def get_metric_dimension(googleanalytics):
    try:
        return yaml.load(open(googleanalytics.metric_dimension_path), Loader=yaml.FullLoader)
    except Exception as e:
        print("Error with metric_dimension_file : %s" % str(e))
        exit()



from setuptools import setup, find_packages

with open('README.rst', 'r') as f:
    readme = f.read()

setup(
    name='pyganalytics',
    version='0.1.15',
    description='Easily get data from Google Analytics',
    long_description=readme,
    long_description_content_type="text/markdown",
    author='Dacker',
    author_email='hello@dacker.co',
    url='https://github.com/dacker-team/pyganalytics',
    keywords='get data google analytics easy',
    packages=find_packages(exclude=('tests', 'docs')),
    package_data={'': ["requirements.txt"]},
    python_requires='>=3',
    install_requires=[
        "dbstream>=0.0.12",
        "PyYAML>=5.1",
        "isoweek>=1.3.3",
        "googleauthentication>=0.0.10"
    ],
)



from dbstream import DBStream
from googleauthentication import GoogleAuthentication


class MetaGoogleAnalytics:
    def __init__(self, googleauthentication: GoogleAuthentication, metric_dimension_path, dbstream: DBStream,
                 mapping_path=None):
        self.googleauthentication = googleauthentication
        self.metric_dimension_path = metric_dimension_path
        self.dbstream = dbstream
        self.mapping_path = mapping_path



def set_dimensions(dimensions):
    """
    Set properly dimensions that we want to load
    @dimensions: list
    """
    result = []
    for i in dimensions:
        d = {
            'name': i
        }
        result.append(d)
    return result


def set_metrics(metrics):
    """
    Set properly metrics that we want to load
    @metrics: list
    """
    result = []
    for i in metrics:
        d = {
            'expression': i
        }
        result.append(d)
    return result


def set_date_range(start_date, end_date):
    """
    Set properly date range
    @start_date: string date "yyyy-mm-dd"
    @end_date: string date "yyyy-mm-dd"
    """
    return [{'startDate': start_date, 'endDate': end_date}]


def get_report(analytics, view_id, dimensions, metrics, start_date, end_date, sampling_level="LARGE",
               metric_filter=None, dimension_filter=None, segments=None,page_token=None):
    """
    Use the Analytics Service Object to query the Analytics Reporting API V4.
    @analytics: result of initialize_api function
    @view_id: Id of Customer's Google Analytics View
    @dimensions: list of dimensions (set at the top of the script)
    @metrics: list of metrics (set at the top of the script)
    @start_date: string date "yyyy-mm-dd"
    @end_date: string date "yyyy-mm-dd"
    @samplingLevel : samplingLevel, "LARGE" by default

    return : API response
    """
    body = {
        'reportRequests': [
            {
                'viewId': view_id,
                'dateRanges': set_date_range(start_date, end_date),
                'dimensions': set_dimensions(dimensions),
                'metrics': set_metrics(metrics),
                'samplingLevel': sampling_level,
                "pageToken": page_token
            }]
    }
    if metric_filter:
        body["reportRequests"][0]["metricFilterClauses"] = metric_filter
    if dimension_filter:
        body["reportRequests"][0]["dimensionFilterClauses"] = dimension_filter
    if segments:
        body["reportRequests"][0]["segments"] = segments
    response = analytics.reports().batchGet(body=body).execute()
    return response
